package sun.font;

import java.util.Locale;
import java.awt.geom.AffineTransform;
import java.awt.Font;
import java.lang.ref.SoftReference;
import java.lang.ref.Reference;
import java.util.concurrent.ConcurrentHashMap;
import java.awt.font.FontRenderContext;

public abstract class Font2D
{
    public static final int FONT_CONFIG_RANK = 2;
    public static final int JRE_RANK = 2;
    public static final int TTF_RANK = 3;
    public static final int TYPE1_RANK = 4;
    public static final int NATIVE_RANK = 5;
    public static final int UNKNOWN_RANK = 6;
    public static final int DEFAULT_RANK = 4;
    private static final String[] boldNames;
    private static final String[] italicNames;
    private static final String[] boldItalicNames;
    private static final FontRenderContext DEFAULT_FRC;
    public Font2DHandle handle;
    protected String familyName;
    protected String fullName;
    protected int style;
    protected FontFamily family;
    protected int fontRank;
    protected CharToGlyphMapper mapper;
    protected ConcurrentHashMap<FontStrikeDesc, Reference> strikeCache;
    protected Reference lastFontStrike;
    public static final int FWIDTH_NORMAL = 5;
    public static final int FWEIGHT_NORMAL = 400;
    public static final int FWEIGHT_BOLD = 700;
    
    public Font2D() {
        this.style = 0;
        this.fontRank = 4;
        this.strikeCache = new ConcurrentHashMap();
        this.lastFontStrike = new SoftReference(null);
    }
    
    public int getStyle() {
        return this.style;
    }
    
    protected void setStyle() {
        String lowerCase = this.fullName.toLowerCase();
        int i = 0;
        while (i < Font2D.boldItalicNames.length) {
            if (lowerCase.indexOf(Font2D.boldItalicNames[i]) != -1) {
                this.style = 3;
                return;
            }
            ++i;
        }
        int j = 0;
        while (j < Font2D.italicNames.length) {
            if (lowerCase.indexOf(Font2D.italicNames[j]) != -1) {
                this.style = 2;
                return;
            }
            ++j;
        }
        int k = 0;
        while (k < Font2D.boldNames.length) {
            if (lowerCase.indexOf(Font2D.boldNames[k]) != -1) {
                this.style = 1;
                return;
            }
            ++k;
        }
    }
    
    public int getWidth() {
        return 5;
    }
    
    public int getWeight() {
        if ((this.style & 0x1) != 0x0) {
            return 700;
        }
        return 400;
    }
    
    int getRank() {
        return this.fontRank;
    }
    
    void setRank(int fontRank) {
        this.fontRank = fontRank;
    }
    
    abstract CharToGlyphMapper getMapper();
    
    protected int getValidatedGlyphCode(int missingGlyphCode) {
        if (missingGlyphCode < 0 || missingGlyphCode >= this.getMapper().getNumGlyphs()) {
            missingGlyphCode = this.getMapper().getMissingGlyphCode();
        }
        return missingGlyphCode;
    }
    
    abstract FontStrike createStrike(FontStrikeDesc p0);
    
    public FontStrike getStrike(Font font) {
        FontStrike fontStrike = (FontStrike)this.lastFontStrike.get();
        if (fontStrike != null) {
            return fontStrike;
        }
        return this.getStrike(font, Font2D.DEFAULT_FRC);
    }
    
    public FontStrike getStrike(Font font, AffineTransform affineTransform, int n, int n2) {
        double n3 = (double)font.getSize2D();
        AffineTransform affineTransform2 = (AffineTransform)affineTransform.clone();
        affineTransform2.scale(n3, n3);
        if (font.isTransformed()) {
            affineTransform2.concatenate(font.getTransform());
        }
        if (affineTransform2.getTranslateX() != 0.0 || affineTransform2.getTranslateY() != 0.0) {
            affineTransform2.setTransform(affineTransform2.getScaleX(), affineTransform2.getShearY(), affineTransform2.getShearX(), affineTransform2.getScaleY(), 0.0, 0.0);
        }
        return this.getStrike(new FontStrikeDesc(affineTransform, affineTransform2, font.getStyle(), n, n2), false);
    }
    
    public FontStrike getStrike(Font font, AffineTransform affineTransform, AffineTransform affineTransform2, int n, int n2) {
        return this.getStrike(new FontStrikeDesc(affineTransform, affineTransform2, font.getStyle(), n, n2), false);
    }
    
    public FontStrike getStrike(Font font, FontRenderContext fontRenderContext) {
        AffineTransform transform = fontRenderContext.getTransform();
        double n = (double)font.getSize2D();
        transform.scale(n, n);
        if (font.isTransformed()) {
            transform.concatenate(font.getTransform());
            if (transform.getTranslateX() != 0.0 || transform.getTranslateY() != 0.0) {
                transform.setTransform(transform.getScaleX(), transform.getShearY(), transform.getShearX(), transform.getScaleY(), 0.0, 0.0);
            }
        }
        return this.getStrike(new FontStrikeDesc(fontRenderContext.getTransform(), transform, font.getStyle(), FontStrikeDesc.getAAHintIntVal(this, font, fontRenderContext), FontStrikeDesc.getFMHintIntVal(fontRenderContext.getFractionalMetricsHint())), false);
    }
    
    FontStrike getStrike(FontStrikeDesc fontStrikeDesc) {
        return this.getStrike(fontStrikeDesc, true);
    }
    
    private FontStrike getStrike(FontStrikeDesc fontStrikeDesc, boolean b) {
        FontStrike fontStrike = (FontStrike)this.lastFontStrike.get();
        if (fontStrike != null && fontStrikeDesc.equals(fontStrike.desc)) {
            return fontStrike;
        }
        Reference<Object> reference = (Reference)this.strikeCache.get(fontStrikeDesc);
        if (reference != null) {
            FontStrike referent = (FontStrike)reference.get();
            if (referent != null) {
                this.lastFontStrike = new SoftReference(referent);
                StrikeCache.refStrike(referent);
                return referent;
            }
        }
        if (b) {
            fontStrikeDesc = new FontStrikeDesc(fontStrikeDesc);
        }
        FontStrike strike = this.createStrike(fontStrikeDesc);
        int type = fontStrikeDesc.glyphTx.getType();
        Reference<Object> value;
        if (type == 32 || ((type & 0x10) != 0x0 && this.strikeCache.size() > 10)) {
            value = StrikeCache.getStrikeRef(strike, true);
        }
        else {
            value = StrikeCache.getStrikeRef(strike);
        }
        this.strikeCache.put(fontStrikeDesc, value);
        this.lastFontStrike = new SoftReference(strike);
        StrikeCache.refStrike(strike);
        return strike;
    }
    
    void removeFromCache(FontStrikeDesc fontStrikeDesc) {
        Reference<Object> reference = (Reference)this.strikeCache.get(fontStrikeDesc);
        if (reference != null && reference.get() == null) {
            this.strikeCache.remove(fontStrikeDesc);
        }
    }
    
    public void getFontMetrics(Font font, AffineTransform affineTransform, Object o, Object o2, float[] array) {
        StrikeMetrics fontMetrics = this.getStrike(font, affineTransform, FontStrikeDesc.getAAHintIntVal(o, this, font.getSize()), FontStrikeDesc.getFMHintIntVal(o2)).getFontMetrics();
        array[0] = fontMetrics.getAscent();
        array[1] = fontMetrics.getDescent();
        array[2] = fontMetrics.getLeading();
        array[3] = fontMetrics.getMaxAdvance();
        this.getStyleMetrics(font.getSize2D(), array, 4);
    }
    
    public void getStyleMetrics(float n, float[] array, int n2) {
        array[n2] = -array[0] / 2.5f;
        array[n2 + 1] = n / 12.0f;
        array[n2 + 2] = array[n2 + 1] / 1.5f;
        array[n2 + 3] = array[n2 + 1];
    }
    
    public void getFontMetrics(Font font, FontRenderContext fontRenderContext, float[] array) {
        StrikeMetrics fontMetrics = this.getStrike(font, fontRenderContext).getFontMetrics();
        array[0] = fontMetrics.getAscent();
        array[1] = fontMetrics.getDescent();
        array[2] = fontMetrics.getLeading();
        array[3] = fontMetrics.getMaxAdvance();
    }
    
    protected byte[] getTableBytes(int n) {
        return null;
    }
    
    protected long getLayoutTableCache() {
        return 0L;
    }
    
    protected long getUnitsPerEm() {
        return 2048L;
    }
    
    boolean supportsEncoding(String s) {
        return false;
    }
    
    public boolean canDoStyle(int n) {
        return n == this.style;
    }
    
    public boolean useAAForPtSize(int n) {
        return true;
    }
    
    public boolean hasSupplementaryChars() {
        return false;
    }
    
    public String getPostscriptName() {
        return this.fullName;
    }
    
    public String getFontName(Locale locale) {
        return this.fullName;
    }
    
    public String getFamilyName(Locale locale) {
        return this.familyName;
    }
    
    public int getNumGlyphs() {
        return this.getMapper().getNumGlyphs();
    }
    
    public int charToGlyph(int n) {
        return this.getMapper().charToGlyph(n);
    }
    
    public int getMissingGlyphCode() {
        return this.getMapper().getMissingGlyphCode();
    }
    
    public boolean canDisplay(char c) {
        return this.getMapper().canDisplay(c);
    }
    
    public boolean canDisplay(int n) {
        return this.getMapper().canDisplay(n);
    }
    
    public byte getBaselineFor(char c) {
        return 0;
    }
    
    public float getItalicAngle(Font font, AffineTransform affineTransform, Object o, Object o2) {
        StrikeMetrics fontMetrics = this.getStrike(font, affineTransform, FontStrikeDesc.getAAHintIntVal(o, this, 12), FontStrikeDesc.getFMHintIntVal(o2)).getFontMetrics();
        if (fontMetrics.ascentY == 0.0f || fontMetrics.ascentX == 0.0f) {
            return 0.0f;
        }
        return fontMetrics.ascentX / -fontMetrics.ascentY;
    }
    
    static {
        boldNames = new String[] { "bold", "demibold", "demi-bold", "demi bold", "negreta", "demi" };
        italicNames = new String[] { "italic", "cursiva", "oblique", "inclined" };
        boldItalicNames = new String[] { "bolditalic", "bold-italic", "bold italic", "boldoblique", "bold-oblique", "bold oblique", "demibold italic", "negreta cursiva", "demi oblique" };
        DEFAULT_FRC = new FontRenderContext(null, false, false);
    }
}
